package cn.iocoder.yudao.module.infra.service.minio;

import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.infra.config.MinioConfigProperties;
import io.minio.BucketExistsArgs;
import io.minio.GetPresignedObjectUrlArgs;
import io.minio.MakeBucketArgs;
import io.minio.MinioClient;
import io.minio.PostPolicy;
import io.minio.PutObjectArgs;
import io.minio.RemoveObjectArgs;
import io.minio.StatObjectArgs;
import io.minio.StatObjectResponse;
import io.minio.http.Method;
import io.minio.ComposeObjectArgs;
import io.minio.ComposeSource;
import io.minio.RemoveObjectsArgs;
import io.minio.messages.DeleteObject;
import io.minio.messages.DeleteError;
import io.minio.Result;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.MultipartUploadCompleteRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.io.InputStream;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

/**
 * MinIO 文件服务类
 *
 * @author AI Assistant
 */
@Slf4j
@Service
public class MinioService {

    @Resource
    private MinioConfigProperties minioConfigProperties;

    private MinioClient minioClient;

    // 用于并行处理的线程池
    private ExecutorService executorService;

    /**
     * 初始化 MinIO 客户端
     */
    @PostConstruct
    public void init() {
        try {
            this.minioClient = MinioClient.builder()
                    .endpoint(minioConfigProperties.getEndpoint())
                    .credentials(minioConfigProperties.getAccessKey(), minioConfigProperties.getSecretKey())
                    .build();

            // 初始化线程池，用于并行处理（使用配置化的线程池大小）
            int corePoolSize = minioConfigProperties.getMultipart().getCorePoolSize();
            int maxPoolSize = minioConfigProperties.getMultipart().getMaxPoolSize();
            this.executorService = Executors.newFixedThreadPool(maxPoolSize, r -> {
                Thread t = new Thread(r, "minio-parallel-pool");
                t.setDaemon(true);
                t.setPriority(Thread.NORM_PRIORITY + 1); // 提高优先级
                return t;
            });

            // 检查桶是否存在，不存在则创建
            String[] bucketsToCreate = {minioConfigProperties.getBucketName(), "user-uploads"};

            for (String bucketName : bucketsToCreate) {
                boolean bucketExists = minioClient.bucketExists(
                        BucketExistsArgs.builder()
                                .bucket(bucketName)
                                .build()
                );

                if (!bucketExists) {
                    minioClient.makeBucket(
                            MakeBucketArgs.builder()
                                    .bucket(bucketName)
                                    .build()
                    );
                    log.info("创建存储桶: {}", bucketName);
                }
            }

            log.info("MinIO 客户端初始化成功. Endpoint: {}, Bucket: {}",
                    minioConfigProperties.getEndpoint(), minioConfigProperties.getBucketName());
        } catch (Exception e) {
            log.error("MinIO 客户端初始化失败", e);
            throw new RuntimeException("MinIO 客户端初始化失败", e);
        }
    }

    /**
     * 销毁方法，关闭线程池
     */
    @PreDestroy
    public void destroy() {
        if (executorService != null && !executorService.isShutdown()) {
            executorService.shutdown();
            log.info("MinIO 线程池已关闭");
        }
    }

    /**
     * 上传单个文件
     *
     * @param file 上传的文件
     * @param path 文件存储路径，如果为空则自动生成
     * @return 文件访问URL
     */
    public String uploadFile(MultipartFile file, String path) throws Exception {
        if (file.isEmpty()) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        // 如果路径为空，则生成随机路径
        if (path == null || path.trim().isEmpty()) {
            String originalFilename = file.getOriginalFilename();
            String extension = "";
            if (originalFilename != null && originalFilename.contains(".")) {
                extension = originalFilename.substring(originalFilename.lastIndexOf("."));
            }
            path = UUID.randomUUID().toString() + extension;
        }

        try (InputStream inputStream = file.getInputStream()) {
            // 上传文件到 MinIO
            minioClient.putObject(
                    PutObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .stream(inputStream, file.getSize(), -1)
                            .contentType(file.getContentType())
                            .build()
            );

            // 生成文件访问URL
            String fileUrl = minioConfigProperties.getEndpoint() + "/" +
                           minioConfigProperties.getBucketName() + "/" + path;

            log.info("文件上传成功: {}", fileUrl);
            return fileUrl;
        } catch (Exception e) {
            log.error("文件上传失败: {}", path, e);
            throw new Exception("文件上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 上传多个文件
     *
     * @param files 上传的文件数组
     * @return 文件访问URL列表
     */
    public List<String> uploadFiles(MultipartFile[] files) throws Exception {
        if (files == null || files.length == 0) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        List<String> fileUrls = new ArrayList<>();

        for (MultipartFile file : files) {
            if (!file.isEmpty()) {
                String fileUrl = uploadFile(file, null);
                fileUrls.add(fileUrl);
            }
        }

        log.info("批量文件上传完成，共上传 {} 个文件", fileUrls.size());
        return fileUrls;
    }

    /**
     * 删除文件
     *
     * @param path 文件路径
     */
    public void deleteFile(String path) throws Exception {
        try {
            minioClient.removeObject(
                    RemoveObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .build()
            );
            log.info("文件删除成功: {}", path);
        } catch (Exception e) {
            log.error("文件删除失败: {}", path, e);
            throw new Exception("文件删除失败: " + e.getMessage(), e);
        }
    }

    /**
     * 生成预签名上传凭证
     *
     * @param fileName 文件名
     * @param fileType 文件类型
     * @return 预签名上传表单数据
     */
    public Map<String, Object> generatePresignedUploadCredentials(String fileName, String fileType) {
        try {
            // 1. 安全验证 - 从SecurityContext获取当前用户
            Long userId = SecurityFrameworkUtils.getLoginUserId();
            if (userId == null) {
                throw new IllegalStateException("用户未登录");
            }

            // 2. 生成唯一对象名
            String sanitizedFileName = sanitizeFilename(fileName);
            String objectName = String.format("user_%d/%s_%s", userId, UUID.randomUUID(), sanitizedFileName);

            // 3. 创建PostPolicy
            // 使用固定的 user-uploads bucket 用于用户上传
            String bucketName = "user-uploads";
            PostPolicy policy = new PostPolicy(bucketName, ZonedDateTime.now().plusHours(1));

            // 设置对象名
            policy.addEqualsCondition("key", objectName);

            // 添加文件大小限制：最小1字节，最大10GB
            policy.addContentLengthRangeCondition(1, 10L * 1024 * 1024 * 1024);

            // 添加Content-Type条件
            policy.addEqualsCondition("Content-Type", fileType);

            // 4. 调用MinIO客户端获取预签名表单数据
            Map<String, String> formData = minioClient.getPresignedPostFormData(policy);

            // 5. 构建返回结构
            Map<String, Object> result = new java.util.HashMap<>();
            result.put("url", minioConfigProperties.getEndpoint() + "/" + bucketName);
            result.put("fields", formData);
            result.put("objectName", objectName);

            log.info("生成预签名上传凭证成功，用户ID: {}, 对象名: {}", userId, objectName);
            return result;

        } catch (Exception e) {
            log.error("生成预签名上传凭证失败", e);
            throw new RuntimeException("生成预签名上传凭证失败: " + e.getMessage(), e);
        }
    }

    /**
     * 文件名安全处理
     *
     * @param fileName 原文件名
     * @return 安全的文件名
     */
    private String sanitizeFilename(String fileName) {
        if (fileName == null || fileName.trim().isEmpty()) {
            return "unknown";
        }
        return fileName.replaceAll("[^a-zA-Z0-9.-]", "_");
    }

    /**
     * 生成预签名下载URL
     *
     * @param objectName 对象名称
     * @return 包含下载URL等信息的Map
     */
    public Map<String, Object> generatePresignedDownloadUrl(String objectName) {
        try {
            // 1. 参数验证
            if (objectName == null || objectName.trim().isEmpty()) {
                throw new IllegalArgumentException("对象名称不能为空");
            }
            // 3. 验证文件是否存在
            String bucketName = "user-uploads";
            try {
                StatObjectResponse stat = minioClient.statObject(
                    StatObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .build()
                );
                log.debug("文件存在检查通过: {}, 大小: {}", objectName, stat.size());
            } catch (Exception e) {
                log.warn("文件不存在: {}", objectName);
                throw new RuntimeException("文件不存在");
            }

            // 4. 生成预签名下载URL
            String downloadUrl = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                    .method(Method.GET)
                    .bucket(bucketName)
                    .object(objectName)
                    .build()
            );

            // 6. 构建返回结果
            Map<String, Object> result = new java.util.HashMap<>();
            result.put("objectName", objectName);
            result.put("downloadUrl", downloadUrl);

            log.info("生成预签名下载URL成功，对象名: {}", objectName);
            return result;

        } catch (Exception e) {
            log.error("生成预签名下载URL失败，对象名: {}", objectName, e);
            throw new RuntimeException("生成预签名下载URL失败: " + e.getMessage(), e);
        }
    }

    /**
     * 初始化分片上传（简化实现）
     * 注意：此实现仅做会话记录，实际分片由前端直接上传到MinIO
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @return 上传会话ID
     */
    public String initMultipartUpload(String uploadId, String objectName) throws Exception {
        try {
            // 验证bucket是否存在
            String bucketName = "user-uploads";
            boolean bucketExists = minioClient.bucketExists(
                BucketExistsArgs.builder()
                    .bucket(bucketName)
                    .build()
            );

            if (!bucketExists) {
                throw new Exception("存储桶不存在: " + bucketName);
            }

            log.info("初始化分片上传成功，上传会话ID: {}, 对象名: {}", uploadId, objectName);
            return uploadId;
        } catch (Exception e) {
            log.error("初始化分片上传失败", e);
            throw new Exception("初始化分片上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 生成分片预签名URL
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param partNumber 分片编号
     * @param expires 有效期（秒）
     * @return 预签名URL
     */
    public String generateChunkPresignedUrl(String uploadId, String objectName, Integer partNumber, Integer expires) throws Exception {
        try {
            // 使用固定的 user-uploads bucket
            String bucketName = "user-uploads";

            // 生成临时分片对象名
            String chunkObjectName = objectName + ".part" + partNumber;

            String presignedUrl = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                    .method(Method.PUT)
                    .bucket(bucketName)
                    .object(chunkObjectName)
                    .expiry(expires)
                    .build()
            );

            log.info("生成分片预签名URL成功，分片编号: {}", partNumber);
            return presignedUrl;
        } catch (Exception e) {
            log.error("生成分片预签名URL失败", e);
            throw new Exception("生成分片预签名URL失败: " + e.getMessage(), e);
        }
    }

    /**
     * 完成分片上传 - 高效合并 + 异步清理
     * 优化合并效率，提高接口响应速度
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param parts 分片信息列表
     * @return 完成结果
     */
    public MultipartUploadCompleteResult completeMultipartUpload(String uploadId, String objectName,
                                                                java.util.List<MultipartUploadCompleteRequest.PartInfo> parts) throws Exception {
        long startTime = System.currentTimeMillis();
        String bucketName = "user-uploads";

        try {
            log.info("开始高效合并{}个分片文件", parts.size());

            // 1. 预处理：排序分片
            parts.sort(Comparator.comparing(MultipartUploadCompleteRequest.PartInfo::getPartNumber));

            // 2. 构建合并源列表
            List<ComposeSource> sources = parts.stream()
                .map(part -> ComposeSource.builder()
                    .bucket(bucketName)
                    .object(objectName + ".part" + part.getPartNumber())
                    .build())
                .collect(Collectors.toList());

            // 3. 智能合并策略 - 根据分片数量选择最优策略
            executeSmartCompose(bucketName, objectName, sources);

            long composeTime = System.currentTimeMillis();
            log.info("合并完成，耗时: {}ms", composeTime - startTime);

            // 4. 并行获取文件信息和URL（同步）
            CompletableFuture<String> urlFuture = CompletableFuture.supplyAsync(() -> {
                try {
                    return minioClient.getPresignedObjectUrl(
                        GetPresignedObjectUrlArgs.builder()
                            .method(Method.GET)
                            .bucket(bucketName)
                            .object(objectName)
                            .build()
                    );
                } catch (Exception e) {
                    log.warn("生成预签名URL失败，使用基本URL", e);
                    return minioConfigProperties.getEndpoint() + "/" + bucketName + "/" + objectName;
                }
            }, executorService);

            CompletableFuture<StatObjectResponse> statFuture = CompletableFuture.supplyAsync(() -> {
                try {
                    return minioClient.statObject(
                        StatObjectArgs.builder()
                            .bucket(bucketName)
                            .object(objectName)
                            .build()
                    );
                } catch (Exception e) {
                    throw new RuntimeException("获取文件信息失败", e);
                }
            }, executorService);

            // 5. 清理分片文件 - 可配置同步/异步模式
            boolean useSyncCleanup = minioConfigProperties.getMultipart().isEnableSyncCleanup();
            
            if (useSyncCleanup) {
                log.info("使用同步清理模式（调试模式）");
                try {
                    cleanupChunkFilesBatchSync(bucketName, objectName, parts);
                } catch (Exception cleanupException) {
                    log.error("同步清理失败，但不影响主流程", cleanupException);
                }
            } else {
                log.info("使用异步清理模式（生产模式）");
                cleanupChunkFilesBatch(bucketName, objectName, parts);
            }

            // 6. 等待URL和文件信息生成（最多3秒）
            String fileUrl = urlFuture.get(3, TimeUnit.SECONDS);
            StatObjectResponse stat = statFuture.get(3, TimeUnit.SECONDS);

            long totalTime = System.currentTimeMillis() - startTime;
            log.info("高效合并完成，文件大小: {}, 总耗时: {}ms", stat.size(), totalTime);

            return new MultipartUploadCompleteResult(fileUrl, stat.etag());

        } catch (Exception e) {
            log.error("高效合并失败", e);
            throw new Exception("分片上传完成失败: " + e.getMessage(), e);
        }
    }

    /**
     * 智能合并策略 - 根据分片数量自动选择最优方案
     * 优化点：
     * 1. 小文件直接合并，避免创建临时文件
     * 2. 大文件使用优化的分批策略
     * 3. 移除不必要的验证步骤
     */
    private void executeSmartCompose(String bucketName, String objectName, List<ComposeSource> sources) throws Exception {
        int sourceCount = sources.size();
        int batchSize = minioConfigProperties.getMultipart().getBatchSize();
        
        log.info("启用智能合并策略，总分片数: {}, 配置批次大小: {}", sourceCount, batchSize);

        // 优化1：小文件直接合并，避免不必要的分批
        if (sourceCount <= batchSize) {
            log.info("分片数量({})不超过批次大小({})，采用直接合并", sourceCount, batchSize);
            
            minioClient.composeObject(
                ComposeObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .sources(sources)
                    .build()
            );
            
            log.info("直接合并完成");
            return;
        }

        // 优化2：大文件使用高效分批策略
        log.info("分片数量({})超过批次大小({})，采用高效分批合并", sourceCount, batchSize);
        executeOptimizedBatchCompose(bucketName, objectName, sources);
    }

    /**
     * 优化的分批并行合并策略
     * 优化点：
     * 1. 移除不必要的临时文件验证
     * 2. 优化批次大小策略
     * 3. 异步清理临时文件，不阻塞主流程
     */
    private void executeOptimizedBatchCompose(String bucketName, String objectName, List<ComposeSource> sources) throws Exception {
        int batchSize = minioConfigProperties.getMultipart().getBatchSize();
        
        // 优化3：动态调整批次大小，大文件使用更大的批次
        int optimizedBatchSize = batchSize;
        
        if (minioConfigProperties.getMultipart().isEnableDynamicBatchSize()) {
            int maxDynamicBatchSize = minioConfigProperties.getMultipart().getMaxDynamicBatchSize();
            optimizedBatchSize = Math.max(batchSize, Math.min(maxDynamicBatchSize, sources.size() / 4));
            
            if (optimizedBatchSize != batchSize) {
                log.info("启用动态批次大小优化：{} -> {}", batchSize, optimizedBatchSize);
            }
        }
        
        log.info("启用优化分批合并策略，总分片数: {}, 批次大小: {}", sources.size(), optimizedBatchSize);

        // 1. 分批创建临时合并对象
        List<String> tempObjects = new ArrayList<>();
        List<CompletableFuture<Void>> batchFutures = new ArrayList<>();

        for (int i = 0; i < sources.size(); i += optimizedBatchSize) {
            int endIndex = Math.min(i + optimizedBatchSize, sources.size());
            List<ComposeSource> batch = sources.subList(i, endIndex);
            String tempObjectName = objectName + ".temp." + (i / optimizedBatchSize);
            tempObjects.add(tempObjectName);

            // 并行处理每个批次
            CompletableFuture<Void> batchFuture = CompletableFuture.runAsync(() -> {
                try {
                    long batchStartTime = System.currentTimeMillis();

                    minioClient.composeObject(
                        ComposeObjectArgs.builder()
                            .bucket(bucketName)
                            .object(tempObjectName)
                            .sources(batch)
                            .build()
                    );

                    long batchTime = System.currentTimeMillis() - batchStartTime;
                    log.debug("批次合并完成: {}, 耗时: {}ms, 分片数: {}",
                             tempObjectName, batchTime, batch.size());

                } catch (Exception e) {
                    log.error("批次合并失败: {}, 原因: {}", tempObjectName, e.getMessage(), e);
                    throw new RuntimeException("批次合并失败: " + tempObjectName, e);
                }
            }, executorService);

            batchFutures.add(batchFuture);
        }

        // 2. 等待所有批次完成（同步）
        try {
            CompletableFuture.allOf(batchFutures.toArray(new CompletableFuture[0])).get();
            log.info("所有批次合并完成，共{}个批次", tempObjects.size());
        } catch (Exception e) {
            log.error("批次合并过程中出现错误", e);
            // 异步清理已创建的临时对象
            cleanupTempObjects(bucketName, tempObjects);
            throw new Exception("分批合并失败", e);
        }

        // 3. 最终合并临时对象（同步）
        try {
            List<ComposeSource> tempSources = tempObjects.stream()
                .map(tempObj -> ComposeSource.builder()
                    .bucket(bucketName)
                    .object(tempObj)
                    .build())
                .collect(Collectors.toList());

            log.info("开始最终合并，临时对象数: {}", tempObjects.size());

            minioClient.composeObject(
                ComposeObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .sources(tempSources)
                    .build()
            );

            log.info("优化分批合并完成，临时对象数: {}", tempObjects.size());

        } catch (Exception e) {
            log.error("最终合并失败，临时对象: {}", tempObjects, e);
            throw new Exception("最终合并失败: " + e.getMessage(), e);
        } finally {
            // 4. 清理临时对象 - 支持同步/异步模式
            boolean useSyncCleanup = minioConfigProperties.getMultipart().isEnableSyncCleanup();
            
            if (useSyncCleanup) {
                log.info("使用同步清理临时对象模式（调试模式）");
                try {
                    cleanupTempObjectsSync(bucketName, tempObjects);
                } catch (Exception cleanupException) {
                    log.error("同步清理临时对象失败，但不影响主流程", cleanupException);
                }
            } else {
                log.info("使用异步清理临时对象模式（生产模式）");
                cleanupTempObjects(bucketName, tempObjects);
            }
        }
    }

    /**
     * 异步清理临时对象（修复版本）
     * 正确处理MinIO删除结果，增加删除验证
     */
    private void cleanupTempObjects(String bucketName, List<String> tempObjects) {
        if (tempObjects.isEmpty()) {
            return;
        }

        CompletableFuture.runAsync(() -> {
            try {
                log.info("开始异步清理{}个临时对象", tempObjects.size());

                List<DeleteObject> deleteObjects = tempObjects.stream()
                    .map(DeleteObject::new)
                    .collect(Collectors.toList());

                // 批量删除临时对象
                Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                    RemoveObjectsArgs.builder()
                        .bucket(bucketName)
                        .objects(deleteObjects)
                        .build()
                );

                // 检查删除结果
                List<String> failedFiles = new ArrayList<>();
                int successCount = 0;
                
                for (Result<DeleteError> result : results) {
                    try {
                        DeleteError error = result.get();
                        if (error != null) {
                            failedFiles.add(error.objectName());
                            log.warn("删除临时对象失败: {}, 错误: {}", error.objectName(), error.message());
                        }
                    } catch (Exception e) {
                        // 如果没有错误，说明删除成功
                        successCount++;
                    }
                }

                log.info("批量删除临时对象完成，成功: {}个，失败: {}个", successCount, failedFiles.size());

                // 如果有失败的文件，尝试重试
                if (!failedFiles.isEmpty()) {
                    log.warn("有{}个临时对象删除失败，尝试单个删除重试", failedFiles.size());
                    retryDeleteFailedTempObjects(bucketName, failedFiles);
                }

            } catch (Exception e) {
                log.error("异步清理临时对象失败", e);
                // 如果批量删除完全失败，降级为单个删除
                fallbackToSingleDeleteTempObjects(bucketName, tempObjects);
            }
        }, executorService);
    }

    /**
     * 同步清理临时对象（用于调试）
     * 确保删除完成后返回，提供详细的删除结果
     */
    public void cleanupTempObjectsSync(String bucketName, List<String> tempObjects) throws Exception {
        if (tempObjects.isEmpty()) {
            log.info("没有临时对象需要清理");
            return;
        }

        try {
            log.info("开始同步清理{}个临时对象", tempObjects.size());

            List<DeleteObject> deleteObjects = tempObjects.stream()
                .map(tempObj -> {
                    log.debug("准备删除临时对象: {}", tempObj);
                    return new DeleteObject(tempObj);
                })
                .collect(Collectors.toList());

            // 批量删除临时对象
            log.info("执行批量删除临时对象，文件数量: {}", deleteObjects.size());
            Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                RemoveObjectsArgs.builder()
                    .bucket(bucketName)
                    .objects(deleteObjects)
                    .build()
            );

            // 检查删除结果
            List<String> failedFiles = new ArrayList<>();
            
            for (Result<DeleteError> result : results) {
                try {
                    DeleteError error = result.get();
                    if (error != null) {
                        failedFiles.add(error.objectName());
                        log.warn("删除临时对象失败: {}, 错误代码: {}, 错误信息: {}", 
                               error.objectName(), error.code(), error.message());
                    }
                } catch (Exception e) {
                    // 如果没有错误，说明删除成功
                }
            }

            // 计算成功删除的文件
            int totalFiles = deleteObjects.size();
            int successCount = totalFiles - failedFiles.size();
            
            log.info("同步批量删除临时对象完成，总计: {}个，成功: {}个，失败: {}个", 
                   totalFiles, successCount, failedFiles.size());

            // 如果有失败的文件，尝试重试
            if (!failedFiles.isEmpty()) {
                log.warn("有{}个临时对象删除失败，尝试单个删除重试", failedFiles.size());
                retryDeleteFailedTempObjects(bucketName, failedFiles);
            }

            // 验证删除结果
            int remainingFiles = verifyTempObjectsCleanupResult(bucketName, tempObjects);
            if (remainingFiles > 0) {
                log.warn("验证发现仍有{}个临时对象未被删除", remainingFiles);
            } else {
                log.info("验证通过，所有临时对象已清理完成");
            }

        } catch (Exception e) {
            log.error("同步清理临时对象失败", e);
            throw new Exception("同步清理临时对象失败: " + e.getMessage(), e);
        }
    }

    /**
     * 验证临时对象清理结果
     * 检查临时对象是否真的被删除了
     */
    private int verifyTempObjectsCleanupResult(String bucketName, List<String> tempObjects) {
        int remainingFiles = 0;
        
        for (String tempObject : tempObjects) {
            try {
                minioClient.statObject(
                    StatObjectArgs.builder()
                        .bucket(bucketName)
                        .object(tempObject)
                        .build()
                );
                // 如果能stat成功，说明文件仍然存在
                remainingFiles++;
                log.warn("临时对象仍然存在: {}", tempObject);
            } catch (Exception e) {
                // 文件不存在，说明删除成功
                log.debug("临时对象已删除: {}", tempObject);
            }
        }
        
        return remainingFiles;
    }

    /**
     * 重试删除失败的临时对象
     */
    private void retryDeleteFailedTempObjects(String bucketName, List<String> failedFiles) {
        int retrySuccessCount = 0;
        
        for (String fileName : failedFiles) {
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(fileName)
                        .build()
                );
                retrySuccessCount++;
                log.debug("重试删除临时对象成功: {}", fileName);
            } catch (Exception e) {
                log.warn("重试删除临时对象仍然失败: {}, 错误: {}", fileName, e.getMessage());
            }
        }
        
        log.info("重试删除临时对象完成，成功删除{}个失败文件", retrySuccessCount);
    }

    /**
     * 降级为单个删除临时对象（当批量删除失败时）
     */
    private void fallbackToSingleDeleteTempObjects(String bucketName, List<String> tempObjects) {
        log.warn("降级为单个删除临时对象模式");
        tempObjects.parallelStream().forEach(tempObject -> {
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(tempObject)
                        .build()
                );
                log.debug("降级删除临时对象: {}", tempObject);
            } catch (Exception e) {
                log.warn("降级删除临时对象失败: {}", tempObject, e);
            }
        });
    }

    /**
     * 异步批量清理分片文件（修复版本）
     * 正确处理MinIO删除结果，增加删除验证
     */
    private void cleanupChunkFilesBatch(String bucketName, String objectName,
                                       List<MultipartUploadCompleteRequest.PartInfo> parts) {
        CompletableFuture.runAsync(() -> {
            try {
                log.info("开始异步批量清理{}个分片文件", parts.size());

                // 构建待删除对象列表
                List<DeleteObject> deleteObjects = parts.stream()
                    .map(part -> {
                        String chunkObjectName = objectName + ".part" + part.getPartNumber();
                        return new DeleteObject(chunkObjectName);
                    })
                    .collect(Collectors.toList());

                if (deleteObjects.isEmpty()) {
                    log.info("没有分片文件需要清理");
                    return;
                }

                // 批量删除分片文件
                Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                    RemoveObjectsArgs.builder()
                        .bucket(bucketName)
                        .objects(deleteObjects)
                        .build()
                );

                // 检查删除结果
                List<String> failedFiles = new ArrayList<>();
                int successCount = 0;
                
                for (Result<DeleteError> result : results) {
                    try {
                        DeleteError error = result.get();
                        if (error != null) {
                            failedFiles.add(error.objectName());
                            log.warn("删除分片文件失败: {}, 错误: {}", error.objectName(), error.message());
                        }
                    } catch (Exception e) {
                        // 如果没有错误，说明删除成功
                        successCount++;
                    }
                }

                log.info("批量删除分片文件完成，成功: {}个，失败: {}个", successCount, failedFiles.size());

                // 如果有失败的文件，尝试重试
                if (!failedFiles.isEmpty()) {
                    log.warn("有{}个分片文件删除失败，尝试单个删除重试", failedFiles.size());
                    retryDeleteFailedChunks(bucketName, failedFiles);
                }

            } catch (Exception e) {
                log.error("异步批量清理分片文件失败", e);
                // 如果批量删除完全失败，降级为单个删除
                fallbackToSingleDelete(bucketName, objectName, parts);
            }
        }, executorService);
    }

    /**
     * 同步批量清理分片文件（用于调试）
     * 确保删除完成后返回，提供详细的删除结果
     */
    public void cleanupChunkFilesBatchSync(String bucketName, String objectName,
                                          List<MultipartUploadCompleteRequest.PartInfo> parts) throws Exception {
        try {
            log.info("开始同步批量清理{}个分片文件", parts.size());

            // 构建待删除对象列表
            List<DeleteObject> deleteObjects = parts.stream()
                .map(part -> {
                    String chunkObjectName = objectName + ".part" + part.getPartNumber();
                    log.debug("准备删除分片文件: {}", chunkObjectName);
                    return new DeleteObject(chunkObjectName);
                })
                .collect(Collectors.toList());

            if (deleteObjects.isEmpty()) {
                log.info("没有分片文件需要清理");
                return;
            }

            // 批量删除分片文件
            log.info("执行批量删除，文件数量: {}", deleteObjects.size());
            Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                RemoveObjectsArgs.builder()
                    .bucket(bucketName)
                    .objects(deleteObjects)
                    .build()
            );

            // 检查删除结果
            List<String> failedFiles = new ArrayList<>();
            List<String> successFiles = new ArrayList<>();
            
            for (Result<DeleteError> result : results) {
                try {
                    DeleteError error = result.get();
                    if (error != null) {
                        failedFiles.add(error.objectName());
                        log.warn("删除分片文件失败: {}, 错误代码: {}, 错误信息: {}", 
                               error.objectName(), error.code(), error.message());
                    }
                } catch (Exception e) {
                    // 如果没有错误，说明删除成功
                    // 但我们需要知道是哪个文件成功了
                    // 由于MinIO API限制，我们无法直接知道，只能通过对比
                }
            }

            // 计算成功删除的文件
            int totalFiles = deleteObjects.size();
            int successCount = totalFiles - failedFiles.size();
            
            log.info("同步批量删除分片文件完成，总计: {}个，成功: {}个，失败: {}个", 
                   totalFiles, successCount, failedFiles.size());

            // 如果有失败的文件，尝试重试
            if (!failedFiles.isEmpty()) {
                log.warn("有{}个分片文件删除失败，尝试单个删除重试", failedFiles.size());
                retryDeleteFailedChunks(bucketName, failedFiles);
            }

            // 验证删除结果
            int remainingFiles = verifyCleanupResult(bucketName, objectName, parts);
            if (remainingFiles > 0) {
                log.warn("验证发现仍有{}个分片文件未被删除", remainingFiles);
            } else {
                log.info("验证通过，所有分片文件已清理完成");
            }

        } catch (Exception e) {
            log.error("同步批量清理分片文件失败", e);
            throw new Exception("同步清理分片文件失败: " + e.getMessage(), e);
        }
    }

    /**
     * 验证清理结果
     * 检查分片文件是否真的被删除了
     */
    private int verifyCleanupResult(String bucketName, String objectName, 
                                   List<MultipartUploadCompleteRequest.PartInfo> parts) {
        int remainingFiles = 0;
        
        for (MultipartUploadCompleteRequest.PartInfo part : parts) {
            String chunkObjectName = objectName + ".part" + part.getPartNumber();
            try {
                minioClient.statObject(
                    StatObjectArgs.builder()
                        .bucket(bucketName)
                        .object(chunkObjectName)
                        .build()
                );
                // 如果能stat成功，说明文件仍然存在
                remainingFiles++;
                log.warn("分片文件仍然存在: {}", chunkObjectName);
            } catch (Exception e) {
                // 文件不存在，说明删除成功
                log.debug("分片文件已删除: {}", chunkObjectName);
            }
        }
        
        return remainingFiles;
    }

    /**
     * 重试删除失败的分片文件
     */
    private void retryDeleteFailedChunks(String bucketName, List<String> failedFiles) {
        int retrySuccessCount = 0;
        
        for (String fileName : failedFiles) {
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(fileName)
                        .build()
                );
                retrySuccessCount++;
                log.debug("重试删除分片文件成功: {}", fileName);
            } catch (Exception e) {
                log.warn("重试删除分片文件仍然失败: {}, 错误: {}", fileName, e.getMessage());
            }
        }
        
        log.info("重试删除完成，成功删除{}个失败文件", retrySuccessCount);
    }

    /**
     * 取消分片上传 - 清理已上传的分片文件
     * 删除所有已上传的分片文件
     *
     * @param uploadId 上传会话ID
     * @param objectName 对象名称
     * @param totalChunks 总分片数
     */
    public void abortMultipartUpload(String uploadId, String objectName, Integer totalChunks) throws Exception {
        try {
            String bucketName = "user-uploads";

            // 清理可能存在的分片文件
            // 如果有totalChunks信息，则使用精确清理，否则尝试删除最多1000个分片文件
            int maxParts = totalChunks != null ? totalChunks : 1000;
            int cleanedCount = 0;

            for (int partNumber = 1; partNumber <= maxParts; partNumber++) {
                String chunkObjectName = objectName + ".part" + partNumber;

                try {
                    // 检查分片文件是否存在
                    minioClient.statObject(
                        StatObjectArgs.builder()
                            .bucket(bucketName)
                            .object(chunkObjectName)
                            .build()
                    );

                    // 如果存在，则删除
                    minioClient.removeObject(
                        RemoveObjectArgs.builder()
                            .bucket(bucketName)
                            .object(chunkObjectName)
                            .build()
                    );

                    cleanedCount++;
                    log.debug("清理分片文件: {}", chunkObjectName);

                } catch (Exception e) {
                    // 分片文件不存在，跳过
                    log.debug("分片文件不存在，跳过: {}", chunkObjectName);
                }
            }

            log.info("取消分片上传成功，上传会话ID: {}, 对象名: {}, 清理了{}个分片文件",
                     uploadId, objectName, cleanedCount);
        } catch (Exception e) {
            log.error("取消分片上传失败", e);
            throw new Exception("取消分片上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 降级为单个删除（当批量删除失败时）- 同步版本
     */
    private void fallbackToSingleDeleteSync(String bucketName, String objectName,
                                           List<MultipartUploadCompleteRequest.PartInfo> parts) {
        log.warn("使用同步单个删除模式");
        int successCount = 0;
        int failCount = 0;
        
        for (MultipartUploadCompleteRequest.PartInfo part : parts) {
            String chunkObjectName = objectName + ".part" + part.getPartNumber();
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(chunkObjectName)
                        .build()
                );
                successCount++;
                log.debug("同步单个删除分片文件成功: {}", chunkObjectName);
            } catch (Exception e) {
                failCount++;
                log.warn("同步单个删除分片文件失败: {}, 错误: {}", chunkObjectName, e.getMessage());
            }
        }
        
        log.info("同步单个删除完成，成功{}个，失败{}个", successCount, failCount);
    }

    /**
     * 降级为单个删除（当批量删除失败时）- 异步版本
     */
    private void fallbackToSingleDelete(String bucketName, String objectName,
                                       List<MultipartUploadCompleteRequest.PartInfo> parts) {
        log.warn("降级为单个删除模式");
        parts.parallelStream().forEach(part -> {
            String chunkObjectName = objectName + ".part" + part.getPartNumber();
            try {
                minioClient.removeObject(
                    RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(chunkObjectName)
                        .build()
                );
                log.debug("降级删除分片文件: {}", chunkObjectName);
            } catch (Exception e) {
                log.warn("降级删除分片文件失败: {}", chunkObjectName, e);
            }
        });
    }

    /**
     * 手动清理临时文件（用于清理残留文件）
     * 
     * @param objectName 对象名称
     * @param tempFileCount 临时文件数量（可选，如果不知道可以传null）
     * @return 清理结果描述
     */
    public String manualCleanupTempFiles(String objectName, Integer tempFileCount) throws Exception {
        String bucketName = "user-uploads";
        List<String> tempObjects = new ArrayList<>();
        
        try {
            // 如果指定了临时文件数量，生成精确的临时文件列表
            if (tempFileCount != null && tempFileCount > 0) {
                for (int i = 0; i < tempFileCount; i++) {
                    tempObjects.add(objectName + ".temp." + i);
                }
            } else {
                // 如果没有指定数量，尝试查找可能的临时文件（最多检查20个）
                for (int i = 0; i < 20; i++) {
                    String tempObject = objectName + ".temp." + i;
                    try {
                        minioClient.statObject(
                            StatObjectArgs.builder()
                                .bucket(bucketName)
                                .object(tempObject)
                                .build()
                        );
                        // 如果文件存在，添加到清理列表
                        tempObjects.add(tempObject);
                        log.info("发现临时文件: {}", tempObject);
                    } catch (Exception e) {
                        // 文件不存在，停止查找
                        break;
                    }
                }
            }
            
            if (tempObjects.isEmpty()) {
                String message = "没有发现需要清理的临时文件";
                log.info(message);
                return message;
            }
            
            log.info("开始手动清理{}个临时文件: {}", tempObjects.size(), tempObjects);
            
            // 使用同步清理方法确保清理完成
            cleanupTempObjectsSync(bucketName, tempObjects);
            
            String message = String.format("手动清理完成，处理了%d个临时文件", tempObjects.size());
            log.info(message);
            return message;
            
        } catch (Exception e) {
            String errorMessage = "手动清理临时文件失败: " + e.getMessage();
            log.error(errorMessage, e);
            throw new Exception(errorMessage, e);
        }
    }

    /**
     * 分片上传完成结果
     */
    public static class MultipartUploadCompleteResult {
        private final String fileUrl;
        private final String etag;

        public MultipartUploadCompleteResult(String fileUrl, String etag) {
            this.fileUrl = fileUrl;
            this.etag = etag;
        }

        public String getFileUrl() {
            return fileUrl;
        }

        public String getEtag() {
            return etag;
        }
    }

}
