package cn.iocoder.yudao.module.system.service.dept;

import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.framework.datapermission.core.annotation.DataPermission;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptApiDTO;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptListReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 部门 Service 接口
 */
public interface DeptService {

    /**
     * 创建部门
     *
     * @param createReqVO 部门信息
     * @return 部门编号
     */
    Long createDept(DeptSaveReqVO createReqVO);

    /**
     * 更新部门
     *
     * @param updateReqVO 部门信息
     */
    void updateDept(DeptSaveReqVO updateReqVO);

    /**
     * 更新工厂部门
     *
     * @param updateReqVO 更新信息
     */
    void updateFactoryDept(DeptSaveReqVO updateReqVO);

    /**
     * 删除部门
     *
     * @param id 部门编号
     */
    void deleteDept(Long id);

    /**
     * 获得部门信息
     *
     * @param id 部门编号
     * @return 部门信息
     */
    DeptDO getDept(Long id);

    @DataPermission(enable = false)
    DeptDO getDeptNoPermission(Long id);

    List<DeptDO> getDeptListNoPermission(Collection<Long> ids);
    /**
     * 获得部门信息数组
     *
     * @param ids 部门编号数组
     * @return 部门信息数组
     */
    List<DeptDO> getDeptList(Collection<Long> ids);

    /**
     * 筛选部门列表
     *
     * @param reqVO 筛选条件请求 VO
     * @return 部门列表
     */
    List<DeptDO> getDeptList(DeptListReqVO reqVO);

    /**
     * 根据部门类型筛选部门列表
     *
     * @param reqVO 筛选条件请求 VO
     * @return 部门列表
     */
    @DataPermission(enable = false)
    List<DeptDO> getDeptListByType(DeptListReqVO reqVO);

    /**
     * 获得指定编号的部门 Map
     *
     * @param ids 部门编号数组
     * @return 部门 Map
     */
    default Map<Long, DeptDO> getDeptMap(Collection<Long> ids) {
        List<DeptDO> list = getDeptList(ids);
        return CollectionUtils.convertMap(list, DeptDO::getId);
    }

    /**
     * 获得指定部门的所有子部门
     *
     * @param id 部门编号
     * @return 子部门列表
     */
    List<DeptDO> getChildDeptList(Long id);

    /**
     * 获得指定部门的所有子部门 （包含自己）
     *
     * @param id 部门编号
     * @return 子部门列表
     */
    List<DeptDO> getChildDept(Long id);

    /**
     * 获得所有子部门，从缓存中
     *
     * @param id 父部门编号
     * @return 子部门列表
     */
    Set<Long> getChildDeptIdListFromCache(Long id);

    /**
     * 校验部门们是否有效。如下情况，视为无效：
     * 1. 部门编号不存在
     * 2. 部门被禁用
     *
     * @param ids 角色编号数组
     */
    void validateDeptList(Collection<Long> ids);

    /**
     * 根据用户ID 检索该用户所担任负责人的部门list
     */
    List<DeptDO> getDeptByLeaderId(Long leaderUserId);

    /**
     * 获得 部门树
     *
     * @return 部门列表
     */
    List<DeptDO> getDeptTree();

    /**
     * 获得指定部门信息
     *
     * @param factoryId 工厂编号
     * @return 部门信息
     */
    DeptDO getDeptByFactoryId(Long factoryId);

    /**
     * 获取部门类型为公司的部门信息
     *
     * @return 部门信息列表
     */
    List<DeptDO> getCompanyDept(Integer method);

    /**
     * 获取所有工厂部门
     *
     * @return
     */
    List<DeptDO> getFactoryDeptList();

    /**
     * 获取所有工厂部门
     *
     * @return
     */
    List<DeptDO> getFactoryDeptList(Long factoryId);

    /**
     * 根据工厂ids获取部门列表
     *
     * @param factoryIds
     * @return
     */
    List<DeptDO> getDeptListByFactoryIds(Collection<Long> factoryIds);

    /**
     * 获取部门列表
     *
     * @param dto
     * @return
     */
    List<DeptDO> getDeptList(DeptApiDTO dto);

    /**
     * 获取用户所在公司的信息
     *
     * @param userId 用户编号
     * @return 公司信息
     */
    DeptDO getUserCompanyDept(Long userId);

    /**
     * 获取所有可用部门列表
     *
     * @return
     */
    List<DeptDO> getAllList();

    /**
     * 获取指定类型的部门列表
     * @param type 部门类型
     * @return 部门列表
     */
    List<DeptDO> getDeptListByType(String type);

    /**
     * 获取工厂部门
     * @return 部门列表
     */
    List<DeptDO> getFactoryDept();

    /**
     * 获取部门所属公司信息
     * @param deptId 部门编号
     * @return 公司信息
     */
    DeptDO getCompanyByDept(Long deptId);

    /**
     * 获取当前登录用户负责的工厂部门
     *
     * @param id
     * @return
     */
    List<DeptDO> getFactoryDeptByLeaderId(Long id);
}
